==> hyakutyping-0.0.1.tar.gz: hyakutyping-0.0.1/src/ShimonokuPractice.cpp <==
/*!
  \file
  \brief ̋̓͗K

  \author Satofumi KAMIMURA

  $Id$

  \todo ca̐ɕ\
  \todo rݏグ@\̒ǉ
  \todo L[\
  \todo a̓͌ɔƁA̘a̓͂ł
*/

#include "ShimonokuPractice.h"
#include "ShimonokuPractice_uni.h"
#include "PracticeMenu_uni.h"
#include "DrawsDefinition.h"
#include "SystemDefinition.h"
#include "ShimonokuInput.h"
#include "CommonResources.h"
#include "ResultDrawer.h"
#include "TypingRecorder.h"
#include "ShuffleWaka.h"
#include "GuiManager.h"
#include "Layer.h"
#include "BaseEntity.h"
#include "StateMachine.h"
#include "GetTicks.h"
#include "Delay.h"
#include "TextSurface.h"
#include "LabelComponent.h"
#include "SdlUtils.h"
#include "InputHandler.h"
#include "InputEvent.h"
#include "InputReceiveComponent.h"
#include "LockGuard.h"
#include "UtfString.h"

using namespace beego;


struct ShimonokuPractice::pImpl {

  class Schedule : public BaseEntity {
    bool is_terminated;
    StateMachine<Schedule>* state_machine;

  public:
    Schedule(int id, CommonResources* common)
      : BaseEntity(id), is_terminated(false) {
      state_machine = new StateMachine<Schedule>(this);
      state_machine->setCurrentState(FirstState::getObject(common));
    }

    ~Schedule(void) {
      delete state_machine;
    }

    void update(void) {
      state_machine->update();
    }

    void changeState(State<Schedule>* new_state) {
      state_machine->changeState(new_state);
    }

    void setTerminate(void) {
      is_terminated = true;
    }

    bool isTerminated(void) {
      return is_terminated;
    }
  };

  class FirstState : public State<Schedule> {
    CommonResources* common;

    FirstState(CommonResources* common_obj) : common(common_obj) {
    }

  public:
    static FirstState* getObject(CommonResources* common) {
      static FirstState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {
      type->setTerminate();
    }

    void execute(Schedule* type) {
      type->changeState(PrologueState::getObject(common));
    }

    void exit(Schedule* type) {

      // Q[̋L^
      int rand_seed = time(NULL);
      //rand_seed = ShuffleWaka::InvalidSeed; // 
      //rand_seed = 0; // Ȃ

      // â̏ԂVbt
      ShuffleWaka wakaShuffle(ShimonokuTyping, rand_seed);
      wakaShuffle.shuffle(common->waka_order);

      // !!! d_K܂̎
      // !!!
      // wakaShuffle.shuffle();

      common->recorder->recordGame(rand_seed, TypingRecorder::ShimonokuTyping);
    }
  };

  class PrologueState : public State<Schedule> {

    CommonResources* common;
    SDL_mutex* mutex;
    // !!! `܂͕ʃNXɂA肷邩
    Surface title_surface;
    Component title_label;
    Surface subTitle_surface;
    Component subTitle_label;
    size_t first_ticks;

    PrologueState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()),
	title_surface(new TextSurface(common->font,
				      PracticeMenu_shimonoku, TitleSize,
				      Fore, Back, true)),
	title_label(new LabelComponent(title_surface)) {
    }

  public:
    static PrologueState* getObject(CommonResources* common) {
      static PrologueState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {

      // ű^CsOvƕ\
      SDL_Rect position;
      set_SdlRect(&position, centerPosition(title_label, 640/2),
		  middlePosition(title_label, 480/4 + SubOffset));
      title_label->setPosition(&position);

      // uX ṽx쐬
      std::vector<Uint16> num_str;
      char buffer[4];
      sprintf(buffer, "%d", common->practice_num);
      ustrcat(num_str, buffer);
      ustrcat(num_str, ShimonokuPractice_PrologueSubTitle);
      Surface new_surface(new TextSurface(common->font, &num_str[0],
					  LargeSize, Fore, Back, true));
      std::swap(new_surface, subTitle_surface);

      Component new_label(new LabelComponent(subTitle_surface));
      std::swap(new_label, subTitle_label);

      set_SdlRect(&position, centerPosition(subTitle_label, 640/2),
		  topPosition(subTitle_label, 480/4 + MenuOffset) + SubOffset);
      subTitle_label->setPosition(&position);

      // ʉ̍ĐJn
      // !!!

      first_ticks = GetTicks();

      placeComponents();
    }

    void execute(Schedule* type) {

      // ҋ@Ԃo߂ATypingState ֑Jڂ
      // !!!
      // !!! ҋ@Ԃ́Aʉ̍ĐԂ

      // !!! ̑ҋ@Ԃ́Aʉ
      InputEvent input_event;
      common->input_receiver->updateInputEvent(input_event);
      size_t spent_msec = GetTicks() - first_ticks;
      if ((spent_msec >= PrologueMsec) ||
	  (InputEvent::isReleased(input_event, SDLK_RETURN))) {
	type->changeState(TypingState::getObject(common));
      }
    }

    void exit(Schedule* type) {
      // zuR|[lg폜
      removeComponents();

      // ʉ̒~
      // !!!
    }

    void placeComponents(void) {
      LockGuard guard(mutex);
      common->front_layer->push_front(title_label);
      common->front_layer->push_front(subTitle_label);
      common->front_layer->push_front(common->input_receiver);
    }

    void removeComponents(void) {
      LockGuard guard(mutex);
      common->front_layer->remove(title_label);
      common->front_layer->remove(subTitle_label);
      common->front_layer->remove(common->input_receiver);
    }
  };

  class TypingState : public State<Schedule> {
    CommonResources* common;
    SDL_mutex* mutex;
    ShimonokuInput shimonoku_input;
    InputEvent input_event;
    int practice_left;
    bool typing_complete;

    TypingState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()),
	shimonoku_input(ShimonokuTyping), typing_complete(false) {
    }

  public:
    static TypingState* getObject(CommonResources* common) {
      static TypingState obj(common);
      return &obj;
    }

    void setPracticeNum(int num) {
      practice_left = num;
    }

    void removeComponents(void) {
      shimonoku_input.removeComponents();
    }

    void enter(Schedule* type) {
      typing_complete = false;
      shimonoku_input.updateWaka();

      LockGuard guard(mutex);
      //fprintf(stderr, "at TypingState::enter()\n");
      common->front_layer->push_front(common->input_receiver);
    }

    void execute(Schedule* type) {

      // ̋A̋A͌`
      shimonoku_input.drawWaka();

      // ͊Aŝ҂
      common->input_receiver->updateInputEvent(input_event);
      if (typing_complete) {

	// s̓͂҂
	if (! InputEvent::isPressed(input_event, SDLK_RETURN)) {
	  return;
	}

	typing_complete = false;
	if (practice_left <= 0) {
	  // KAResultState ֑Jڂ
	  type->changeState(ResultState::getObject(common));
	} else {
	  // ēxAâ̓͂s
	  shimonoku_input.setNextWaka();
	  type->changeState(TypingState::getObject(common));
	}
	return;
      }

      // ̋̓͗
      if (! typing_complete) {
	shimonoku_input.drawInput();
      }

      // ͂玟̘â
      if (shimonoku_input.isComplete()) {
	--practice_left;
	shimonoku_input.removeInputComponents();
	typing_complete = true;
      }
    }

    void exit(Schedule* type) {
      // zuR|[lg폜
      removeComponents();
    }

    bool textEmpty(void) {
      return shimonoku_input.textEmpty();
    }
  };


  class ResultState : public State<Schedule> {
    CommonResources* common;
    ResultDrawer result_drawer;
    size_t first_ticks;

    ResultState(CommonResources* common_obj)
      : common(common_obj), result_drawer(ShimonokuTyping) {
      // !!!
    }

  public:
    static ResultState* getObject(CommonResources* common) {
      static ResultState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {

      // ^CsOʂ̕]
      // !!! ۂ́A]ƃT[oւ̃f[^MAȂǂH

      // \p^C}[JE^̏
      result_drawer.createResult();
      first_ticks = GetTicks();
    }

    void execute(Schedule* type) {

      // ^CsO̕]\
      size_t ticks = GetTicks() - first_ticks;
      result_drawer.drawResult(ticks);

      // L[͂ȂǂŁAI
      if (result_drawer.keyPressed()) {
	type->changeState(WaitKeyState::getObject(common));
      }
    }

    void exit(Schedule* type) {
    }

    void removeComponents(void) {
      result_drawer.removeComponents();
    }
  };

  // !!! ̃NX́ÂƋʂɂȂ̂ŁA肽
  class WaitKeyState : public State<Schedule> {
    CommonResources* common;
    SDL_mutex* mutex;
    InputEvent input_event;

    WaitKeyState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()) {
    }

  public:
    static WaitKeyState* getObject(CommonResources* common) {
      static WaitKeyState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {
      LockGuard guard(mutex);
      //fprintf(stderr, "at WaitKeyState::enter()\n");
      common->front_layer->push_front(common->input_receiver);
    }

    void execute(Schedule* type) {
      common->input_receiver->updateInputEvent(input_event);
      if (InputEvent::isReleased(input_event, SDLK_RETURN)) {
	type->changeState(FirstState::getObject(common));
      }
    }

    void exit(Schedule* type) {
      removeComponents();
    }

    void removeComponents(void) {
      LockGuard guard(mutex);
      //fprintf(stderr, "at WaitKeyState::removeComponents()\n");
      common->front_layer->remove(common->input_receiver);
    }
  };

  CommonResources* common;
  Schedule scheduler;

  pImpl(void)
    : common(CommonResources::getObject()),
      scheduler(GameSchedulerId, common) {
    TypingState::getObject(common)->setPracticeNum(common->practice_num);
  }

  void removeComponents(void) {
    PrologueState::getObject(common)->removeComponents();
    TypingState::getObject(common)->removeComponents();
    ResultState::getObject(common)->removeComponents();
    WaitKeyState::getObject(common)->removeComponents();
  }
};


ShimonokuPractice::ShimonokuPractice(void) : pimpl(new pImpl) {
}


ShimonokuPractice::~ShimonokuPractice(void) {
}


void ShimonokuPractice::run(void) {

  InputHandler& input = *pimpl->common->input;
  bool quit = false;
  bool escape_pressed = false;
  while (quit == false) {

    // Ԃ̍XV
    pimpl->scheduler.update();

    // I
    quit |= pimpl->scheduler.isTerminated();
    input.update_all();
    if (input.haveQuitEvent()) {
      pimpl->common->front_layer->disable();
      quit |= true;
    }
    if (pImpl::TypingState::getObject(pimpl->common)->textEmpty()) {
      // !!! ̂́AGȎdg݂͂ȂƂȂ...
      escape_pressed |= input.isPressed(SDLK_ESCAPE);
      if (escape_pressed && input.isReleased(SDLK_ESCAPE)) {
	quit |= true;
      }
    }
    delay(1);
  }

  // eR|[lg̍폜
  pimpl->removeComponents();
}

==> hyakutyping-0.0.2.tar.gz: hyakutyping-0.0.2/src/ShimonokuPractice.cpp <==
/*!
  \file
  \brief ̋̓͗K

  \author Satofumi KAMIMURA

  $Id$

  \todo ca̐ɕ\
  \todo rݏグ@\̒ǉ
  \todo L[\
  \todo a̓͌ɔƁA̘a̓͂ł
*/

#include "ShimonokuPractice.h"
#include "ShimonokuPractice_uni.h"
#include "PracticeMenu_uni.h"
#include "DrawsDefinition.h"
#include "SystemDefinition.h"
#include "ShimonokuInput.h"
#include "CommonResources.h"
#include "ResourceDefinition.h"
#include "ResultDrawer.h"
#include "TypingRecorder.h"
#include "ShuffleWaka.h"
#include "GuiManager.h"
#include "Layer.h"
#include "BaseEntity.h"
#include "StateMachine.h"
#include "MusicManager.h"
#include "GetTicks.h"
#include "Delay.h"
#include "TextSurface.h"
#include "LabelComponent.h"
#include "SdlUtils.h"
#include "InputHandler.h"
#include "InputEvent.h"
#include "InputReceiveComponent.h"
#include "LockGuard.h"
#include "UtfString.h"

using namespace beego;


struct ShimonokuPractice::pImpl {

  class Schedule : public BaseEntity {
    bool is_terminated;
    StateMachine<Schedule>* state_machine;

  public:
    Schedule(int id, CommonResources* common)
      : BaseEntity(id), is_terminated(false) {
      state_machine = new StateMachine<Schedule>(this);
      state_machine->setCurrentState(FirstState::getObject(common));
    }

    ~Schedule(void) {
      delete state_machine;
    }

    void update(void) {
      state_machine->update();
    }

    void changeState(State<Schedule>* new_state) {
      state_machine->changeState(new_state);
    }

    void setTerminate(void) {
      is_terminated = true;
    }

    bool isTerminated(void) {
      return is_terminated;
    }
  };

  class FirstState : public State<Schedule> {
    CommonResources* common;

    FirstState(CommonResources* common_obj) : common(common_obj) {
    }

  public:
    static FirstState* getObject(CommonResources* common) {
      static FirstState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {
      type->setTerminate();
    }

    void execute(Schedule* type) {
      type->changeState(PrologueState::getObject(common));
    }

    void exit(Schedule* type) {

      // Q[̋L^
      int rand_seed = time(NULL);
      //rand_seed = ShuffleWaka::InvalidSeed; // 
      //rand_seed = 0; // Ȃ

      // â̏ԂVbt
      ShuffleWaka wakaShuffle(ShimonokuTyping, rand_seed);
      wakaShuffle.shuffle(common->waka_order);

      // !!! d_K܂̎
      // !!!
      // wakaShuffle.shuffle();

      common->recorder->recordGame(rand_seed, TypingRecorder::ShimonokuTyping);
    }
  };

  class PrologueState : public State<Schedule> {

    CommonResources* common;
    SDL_mutex* mutex;
    // !!! `܂͕ʃNXɂA肷邩
    Surface title_surface;
    Component title_label;
    Surface subTitle_surface;
    Component subTitle_label;
    size_t first_ticks;

    PrologueState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()),
	title_surface(new TextSurface(common->font,
				      PracticeMenu_shimonoku, TitleSize,
				      Fore, Back, true)),
	title_label(new LabelComponent(title_surface)) {
    }

  public:
    static PrologueState* getObject(CommonResources* common) {
      static PrologueState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {

      // ű^CsOvƕ\
      SDL_Rect position;
      set_SdlRect(&position, centerPosition(title_label, 640/2),
		  middlePosition(title_label, 480/4 + SubOffset));
      title_label->setPosition(&position);

      // uX ṽx쐬
      std::vector<Uint16> num_str;
      char buffer[4];
      sprintf(buffer, "%d", common->practice_num);
      ustrcat(num_str, buffer);
      ustrcat(num_str, ShimonokuPractice_PrologueSubTitle);
      Surface new_surface(new TextSurface(common->font, &num_str[0],
					  LargeSize, Fore, Back, true));
      std::swap(new_surface, subTitle_surface);

      Component new_label(new LabelComponent(subTitle_surface));
      std::swap(new_label, subTitle_label);

      set_SdlRect(&position, centerPosition(subTitle_label, 640/2),
		  topPosition(subTitle_label, 480/4 + MenuOffset) + SubOffset);
      subTitle_label->setPosition(&position);

      placeComponents();

      if (common->music) {
	// !!! dg݂A܂莚Aʕ\̂ɂKp邱
	// y̒~҂
	while (common->music->nowPlaying()) {
	  delay(10);
	}

	// ʉ̍ĐJn
	// !!!
      }
      first_ticks = GetTicks();
    }

    void execute(Schedule* type) {

      // ҋ@Ԃo߂ATypingState ֑Jڂ
      // !!!
      // !!! ҋ@Ԃ́Aʉ̍ĐԂ

      // !!! ̑ҋ@Ԃ́Aʉ
      InputEvent input_event;
      common->input_receiver->updateInputEvent(input_event);
      size_t spent_msec = GetTicks() - first_ticks;
      if ((spent_msec >= PrologueMsec) ||
	  (InputEvent::isReleased(input_event, SDLK_RETURN))) {
	type->changeState(TypingState::getObject(common));
      }
    }

    void exit(Schedule* type) {
      // zuR|[lg폜
      removeComponents();

      // ʉ̒~
      // !!!

      // y̍Đ
      if (common->music) {
	common->music->setNextMusic(BackMusic_2);
      }
    }

    void placeComponents(void) {
      LockGuard guard(mutex);
      common->front_layer->push_front(title_label);
      common->front_layer->push_front(subTitle_label);
      common->front_layer->push_front(common->input_receiver);
    }

    void removeComponents(void) {
      LockGuard guard(mutex);
      common->front_layer->remove(title_label);
      common->front_layer->remove(subTitle_label);
      common->front_layer->remove(common->input_receiver);
    }
  };

  class TypingState : public State<Schedule> {
    CommonResources* common;
    SDL_mutex* mutex;
    ShimonokuInput shimonoku_input;
    InputEvent input_event;
    int practice_left;
    bool typing_complete;

    TypingState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()),
	shimonoku_input(ShimonokuTyping), typing_complete(false) {
    }

  public:
    static TypingState* getObject(CommonResources* common) {
      static TypingState obj(common);
      return &obj;
    }

    void setPracticeNum(int num) {
      practice_left = num;
    }

    void removeComponents(void) {
      shimonoku_input.removeComponents();
    }

    void enter(Schedule* type) {
      typing_complete = false;
      shimonoku_input.updateWaka();

      LockGuard guard(mutex);
      //fprintf(stderr, "at TypingState::enter()\n");
      common->front_layer->push_front(common->input_receiver);
    }

    void execute(Schedule* type) {

      // ̋A̋A͌`
      shimonoku_input.drawWaka();

      // ͊Aŝ҂
      common->input_receiver->updateInputEvent(input_event);
      if (typing_complete) {

	// s̓͂҂
	if (! InputEvent::isPressed(input_event, SDLK_RETURN)) {
	  return;
	}

	typing_complete = false;
	if (practice_left <= 0) {
	  // KAResultState ֑Jڂ
	  type->changeState(ResultState::getObject(common));
	} else {
	  // ēxAâ̓͂s
	  shimonoku_input.setNextWaka();
	  type->changeState(TypingState::getObject(common));
	}
	return;
      }

      // ̋̓͗
      if (! typing_complete) {
	shimonoku_input.drawInput();
      }

      // ͂玟̘â
      if (shimonoku_input.isComplete()) {
	--practice_left;
	shimonoku_input.removeInputComponents();
	typing_complete = true;
      }
    }

    void exit(Schedule* type) {
      // zuR|[lg폜
      removeComponents();
    }

    bool textEmpty(void) {
      return shimonoku_input.textEmpty();
    }
  };


  class ResultState : public State<Schedule> {
    CommonResources* common;
    ResultDrawer result_drawer;
    size_t first_ticks;

    ResultState(CommonResources* common_obj)
      : common(common_obj), result_drawer(ShimonokuTyping) {
      // !!!
    }

  public:
    static ResultState* getObject(CommonResources* common) {
      static ResultState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {

      // ^CsOʂ̕]
      // !!! ۂ́A]ƃT[oւ̃f[^MAȂǂH

      // \p^C}[JE^̏
      result_drawer.createResult();
      first_ticks = GetTicks();
    }

    void execute(Schedule* type) {

      // ^CsO̕]\
      size_t ticks = GetTicks() - first_ticks;
      result_drawer.drawResult(ticks);

      // L[͂ȂǂŁAI
      if (result_drawer.keyPressed()) {
	type->changeState(WaitKeyState::getObject(common));
      }
    }

    void exit(Schedule* type) {
    }

    void removeComponents(void) {
      result_drawer.removeComponents();
    }
  };

  // !!! ̃NX́ÂƋʂɂȂ̂ŁA肽
  class WaitKeyState : public State<Schedule> {
    CommonResources* common;
    SDL_mutex* mutex;
    InputEvent input_event;

    WaitKeyState(CommonResources* common_obj)
      : common(common_obj), mutex(common->gui->getMutex()) {
    }

  public:
    static WaitKeyState* getObject(CommonResources* common) {
      static WaitKeyState obj(common);
      return &obj;
    }

    void enter(Schedule* type) {
      LockGuard guard(mutex);
      //fprintf(stderr, "at WaitKeyState::enter()\n");
      common->front_layer->push_front(common->input_receiver);
    }

    void execute(Schedule* type) {
      common->input_receiver->updateInputEvent(input_event);
      if (InputEvent::isReleased(input_event, SDLK_RETURN)) {
	type->changeState(FirstState::getObject(common));
      }
    }

    void exit(Schedule* type) {
      removeComponents();
    }

    void removeComponents(void) {
      LockGuard guard(mutex);
      //fprintf(stderr, "at WaitKeyState::removeComponents()\n");
      common->front_layer->remove(common->input_receiver);
    }
  };

  CommonResources* common;
  Schedule scheduler;

  pImpl(void)
    : common(CommonResources::getObject()),
      scheduler(GameSchedulerId, common) {
    TypingState::getObject(common)->setPracticeNum(common->practice_num);
  }

  void removeComponents(void) {
    PrologueState::getObject(common)->removeComponents();
    TypingState::getObject(common)->removeComponents();
    ResultState::getObject(common)->removeComponents();
    WaitKeyState::getObject(common)->removeComponents();
  }
};


ShimonokuPractice::ShimonokuPractice(void) : pimpl(new pImpl) {
}


ShimonokuPractice::~ShimonokuPractice(void) {
}


void ShimonokuPractice::run(void) {

  InputHandler& input = *pimpl->common->input;
  bool quit = false;
  bool escape_pressed = false;
  while (quit == false) {

    // Ԃ̍XV
    pimpl->scheduler.update();

    // I
    quit |= pimpl->scheduler.isTerminated();
    input.update_all();
    if (input.haveQuitEvent()) {
      pimpl->common->front_layer->disable();
      quit |= true;
    }
    if (pImpl::TypingState::getObject(pimpl->common)->textEmpty()) {
      // !!! ̂́AGȎdg݂͂ȂƂȂ...
      escape_pressed |= input.isPressed(SDLK_ESCAPE);
      if (escape_pressed && input.isReleased(SDLK_ESCAPE)) {
	quit |= true;
      }
    }
    delay(1);
  }

  // eR|[lg̍폜
  pimpl->removeComponents();
}
